@@ -22,6 +22,7 @@ from sql_to_ibis.sql_objects import (
     CrossJoin,
     DerivedColumn,
     Expression,
+    GroupByColumn,
     Join,
     JoinBase,
     Literal,
@@ -273,7 +274,7 @@ class SQLTransformer(TransformerBaseClass):
                 match = re.match(r"(?P<type>.*)\souter", join_type)
                 if match:
                     join_type = match.group("type")
-            if join_type in ("full", "cross"):
+            if join_type in {"full", "cross"}:
                 join_type = "outer"
 
         # Check that there is a column from both sides
@@ -328,10 +329,11 @@ class SQLTransformer(TransformerBaseClass):
         query_info.all_names.append(token.final_name)
         query_info.name_order[token.final_name] = token_pos
 
-        if isinstance(token, (Column, Literal, Expression)):
+        if isinstance(token, GroupByColumn):
+            query_info.group_columns.append(token)
+        elif isinstance(token, (Column, Literal, Expression)):
             query_info.columns.append(token)
-
-        if isinstance(token, Aggregate):
+        elif isinstance(token, Aggregate):
             query_info.aggregates[token.final_name] = token
 
     def handle_token_or_tree(self, query_info: QueryInfo, token_or_tree, item_pos):
@@ -345,8 +347,6 @@ class SQLTransformer(TransformerBaseClass):
         if isinstance(token_or_tree, Token):
             if token_or_tree.type == "from_expression":
                 query_info.table_names.append(token_or_tree.value)
-            elif token_or_tree.type == "group":
-                query_info.group_columns.append(token_or_tree.value)
             elif token_or_tree.type == "where_expr":
                 query_info.where_expr = token_or_tree.value
         elif isinstance(token_or_tree, Tree):
@@ -361,13 +361,29 @@ class SQLTransformer(TransformerBaseClass):
         :param select_expressions:
         :return:
         """
-        tables = []
+        tables: List[Token] = []
         having_expr = None
         where_expr = None
         for select_expression in select_expressions:
             if isinstance(select_expression, Tree):
                 if select_expression.data == "from_expression":
-                    tables.append(select_expression.children[0])
+                    table_object = select_expression.children[0]
+                    if isinstance(table_object, JoinBase):
+                        tables += [
+                            table_object.right_table_name,
+                            table_object.left_table_name,
+                        ]
+                    elif (
+                        isinstance(table_object, Tree)
+                        and table_object.data == "cross_join_expression"
+                    ):
+                        cross_join: CrossJoin = table_object.children[0]
+                        tables += [
+                            cross_join.right_table_name,
+                            cross_join.left_table_name,
+                        ]
+                    else:
+                        tables.append(table_object)
                 elif select_expression.data == "having_expr":
                     having_expr = select_expression
                 elif select_expression.data == "where_expr":
@@ -483,85 +499,65 @@ class SQLTransformer(TransformerBaseClass):
                     )
         return having
 
-    def _handle_grouping_vals(
-        self,
-        group_column_names: List[str],
-        table_names: List[str],
-        selected_columns: List[Value],
-    ):
-        group_column_vals = None
-        column_name_lower_to_column = {
-            column.get_name().lower(): column for column in selected_columns
-        }
-        if group_column_names:
-            group_column_names = self._get_true_column_names(
-                group_column_names, table_names
-            )
-            group_column_vals = self._get_columns(group_column_names, table_names)
-            if selected_columns:
-                # If the value was selected and named something, that's what it
-                # should be grouped by if capitalized
-                for i, group_column_name in enumerate(group_column_names):
-                    lower_column_name = group_column_name.lower()
-                    if lower_column_name in column_name_lower_to_column:
-                        column_object = column_name_lower_to_column[lower_column_name]
-                        group_column_vals[i] = group_column_vals[i].name(
-                            column_object.get_name()
-                        )
-        return group_column_names, group_column_vals
-
     def handle_aggregation(
         self,
         aggregates: Dict[str, Aggregate],
-        group_column_names: List[str],
+        group_columns: List[GroupByColumn],
         table: TableExpr,
         having_expr: Tree,
         internal_transformer: InternalTransformer,
         selected_columns: List[Value],
-        table_names: List[str],
     ):
         """
         Handles all aggregation operations when translating from dictionary info
         to dataframe
         """
-        column_name_to_column = {
-            column.get_name().lower(): column for column in selected_columns
+        selected_column_names = {
+            column.get_name().lower() for column in selected_columns
         }
-
         aggregate_ibis_columns = self._get_aggregate_ibis_columns(aggregates)
         having = self._handle_having_expressions(
-            having_expr, internal_transformer, table, aggregates, group_column_names
-        )
-        group_column_names, group_column_vals = self._handle_grouping_vals(
-            group_column_names, table_names, selected_columns
+            having_expr,
+            internal_transformer,
+            table,
+            aggregates,
+            [group_column.get_name() for group_column in group_columns],
         )
 
-        if group_column_names and having is not None and not aggregates:
+        if group_columns and not selected_column_names:
+            for group_column in group_columns:
+                group_column.set_ibis_name_to_name()
+
+        if group_columns and having is not None and not aggregates:
             raise NotImplementedError(
-                "Group by, having, without aggregation not yet " "implemented"
+                "Group by, having, without aggregation not yet implemented"
             )
-
-        if group_column_names and not aggregates:
-            for column in table.columns:
-                if column not in group_column_names:
+        if group_columns and not aggregates:
+            for column in [
+                selected_column.get_name() for selected_column in selected_columns
+            ]:
+                if column not in group_columns:
                     raise InvalidQueryException(
                         self.format_column_needs_agg_or_group_msg(column)
                     )
             table = table.distinct()
-        elif aggregates and not group_column_names:
+        elif aggregates and not group_columns:
             table = table.aggregate(aggregate_ibis_columns, having=having)
-        elif aggregates and group_column_names:
-            table = table.group_by(group_column_vals)
+        elif aggregates and group_columns:
+            table = table.group_by(
+                [group_column.value for group_column in group_columns]
+            )
             if having is not None:
                 table = table.having(having)
             table = table.aggregate(aggregate_ibis_columns)
 
         non_selected_columns = []
-        if group_column_names:
-            for group_column in group_column_names:
-                if group_column.lower() not in column_name_to_column:
-                    non_selected_columns.append(group_column)
-            table = table.drop(non_selected_columns)
+        if group_columns and aggregates:
+            for group_column in group_columns:
+                if group_column.get_name().lower() not in selected_column_names:
+                    non_selected_columns.append(group_column.group_by_name)
+            if non_selected_columns:
+                table = table.drop(non_selected_columns)
 
         return table
 
@@ -602,23 +598,104 @@ class SQLTransformer(TransformerBaseClass):
             return ibis_table.projection(column_mutation)
         return ibis_table
 
-    def handle_join(self, join: JoinBase) -> TableExpr:
+    def _get_table_expr_columns(self, table: TableExpr):
+        return table.get_columns(table.columns)
+
+    def handle_duplicate_columns_in_join(
+        self, right_table: TableExpr, left_table: TableExpr, join: JoinBase
+    ):
+        duplicate_columns = set(left_table.columns).intersection(right_table.columns)
+        for column in duplicate_columns:
+            left_table = left_table.relabel(
+                {column: f"{join.left_table_name}." f"{column}"}
+            )
+            right_table = right_table.relabel(
+                {column: f"{join.right_table_name}" f".{column}"}
+            )
+
+        if isinstance(join, Join) and join.left_on == join.right_on:
+            join.left_on = f"{join.left_table_name}.{join.left_on}"
+            join.right_on = f"{join.right_table_name}.{join.right_on}"
+
+        return left_table, right_table
+
+    @staticmethod
+    def _columns_have_select_star(columns: List[Value]):
+        for column in columns:
+            if column.get_name() == "*":
+                return True
+        return False
+
+    @staticmethod
+    def _rename_duplicates(
+        table: TableExpr, duplicates: Set[str], table_name: str, table_columns: list
+    ):
+        for i, column in enumerate(table.columns):
+            if column in duplicates:
+                table_columns[i] = table_columns[i].name(f"{table_name}.{column}")
+        return table_columns
+
+    def get_all_join_columns_handle_duplicates(
+        self, left: TableExpr, right: TableExpr, join: JoinBase
+    ):
+        left_columns = self._get_table_expr_columns(left)
+        right_columns = self._get_table_expr_columns(right)
+        duplicates = set(left.columns).intersection(right.columns)
+        left_columns = self._rename_duplicates(
+            left, duplicates, join.left_table_name, left_columns
+        )
+        right_columns = self._rename_duplicates(
+            right, duplicates, join.right_table_name, right_columns
+        )
+        return left_columns + right_columns
+
+    def handle_join(self, join: JoinBase, columns: List[Value]) -> TableExpr:
         """
         Return the dataframe and execution plan resulting from a join
         :param join:
+        :param columns: List of all column values
         :return:
         """
-        left_table = self.get_table(join.left_table)
-        right_table = self.get_table(join.right_table)
+        result: TableExpr = None
+        all_columns: List[Value] = []
+        left_table = self.get_table(join.left_table_name)
+        right_table = self.get_table(join.right_table_name)
+
+        if self._columns_have_select_star(columns):
+            all_columns = self.get_all_join_columns_handle_duplicates(
+                left_table, right_table, join
+            )
+
         if isinstance(join, Join):
-            return left_table.join(
+            result = left_table.join(
                 right_table,
                 predicates=left_table.get_column(join.left_on)
                 == right_table.get_column(join.right_on),
                 how=join.join_type,
             )
         if isinstance(join, CrossJoin):
-            return ibis.cross_join(left_table, right_table)
+            result = ibis.cross_join(left_table, right_table)
+
+        if all_columns:
+            return result[all_columns]
+        return result
+
+    def _set_casing_for_groupby_names(
+        self, groupby_columns: List[GroupByColumn], selected_columns: List[Value]
+    ):
+        lower_case_to_true_column_name = {
+            column.get_name().lower(): column.get_name() for column in selected_columns
+        }
+        for groupby_column in groupby_columns:
+            lower_case_group_name = groupby_column.get_name().lower()
+            if lower_case_group_name in lower_case_to_true_column_name:
+                selection_statement_name = lower_case_to_true_column_name[
+                    lower_case_group_name
+                ]
+                groupby_column.group_by_name = selection_statement_name
+                groupby_column.value = groupby_column.value.name(
+                    selection_statement_name
+                )
 
     def to_ibis_table(self, query_info: QueryInfo):
         """
@@ -631,11 +708,37 @@ class SQLTransformer(TransformerBaseClass):
         first_table = self.get_table(table_names[0])
 
         if isinstance(first_table, JoinBase):
-            first_table = self.handle_join(join=first_table)
+            first_table = self.handle_join(join=first_table, columns=query_info.columns)
         for table_name in table_names[1:]:
             next_frame = self.get_table(table_name)
             first_table = first_table.cross_join(next_frame)
 
+        self._set_casing_for_groupby_names(query_info.group_columns, query_info.columns)
+
+        selected_group_columns = []
+        group_column_names = {
+            group_column.group_by_name for group_column in query_info.group_columns
+        }
+        if query_info.group_columns and query_info.aggregates:
+            selection_column_names = [
+                column.get_name() for column in query_info.columns
+            ]
+            remaining_columns = set(selection_column_names) - group_column_names
+            if remaining_columns:
+                raise InvalidQueryException(
+                    f"Must have grouping for columns in {remaining_columns}"
+                )
+            selected_group_columns = [
+                column
+                for column in query_info.columns
+                if column.get_name() in set(group_column_names)
+            ]
+
+            query_info.columns = [
+                column
+                for column in query_info.columns
+                if column.get_name() in remaining_columns
+            ]
         new_table = self.handle_selection(first_table, query_info.columns)
         new_table = self.handle_filtering(
             new_table, query_info.where_expr, query_info.internal_transformer
@@ -646,8 +749,7 @@ class SQLTransformer(TransformerBaseClass):
             new_table,
             query_info.having_expr,
             query_info.internal_transformer,
-            query_info.columns,
-            query_info.table_names,
+            selected_group_columns,
         )
 
         if query_info.distinct:
