@@ -24,6 +24,7 @@ from sql_to_ibis.sql_objects import (
     CrossJoin,
     Date,
     Expression,
+    GroupByColumn,
     JoinBase,
     Literal,
     Number,
@@ -146,7 +147,11 @@ class InternalTransformer(TransformerBaseClass):
     """
 
     def __init__(
-        self, tables, dataframe_map, column_name_map, column_to_dataframe_name
+        self,
+        tables: List[Token],
+        dataframe_map,
+        column_name_map,
+        column_to_dataframe_name,
     ):
         TransformerBaseClass.__init__(
             self, dataframe_map=dataframe_map, column_name_map=column_name_map
@@ -647,14 +652,16 @@ class InternalTransformer(TransformerBaseClass):
         """
         return args[0]
 
-    def group_by(self, column):
+    def group_by(self, columns: List[Column]):
         """
         Returns a group token_or_tree
-        :param column: Column to group by
+        :param columns: Column to group by
         :return: group token_or_tree
         """
-        column = column[0]
-        return Token("group", str(column.name))
+        assert len(columns) == 1
+        column = columns[0]
+        group_by = GroupByColumn.from_column_type(column)
+        return group_by
 
     def as_type(self, column_and_type):
         """
